﻿using Syncfusion.Data;
using Syncfusion.WinForms.DataGrid;
using Syncfusion.WinForms.DataGrid.Enums;
using Syncfusion.WinForms.DataGrid.Renderers;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SfDataGrid_TableSummary
{
    public partial class Form1 : Form
    {
        List<ReceiptDetType> _ReceiptsTypesList { get; set; }
        public Form1()
        {
            InitializeComponent();
            InitializeEditReceiptsGrid();
        }

        private void InitializeEditReceiptsGrid()
        {
            dgvEditReceipts.Columns.Clear();

            Syncfusion.WinForms.DataGrid.Styles.HeaderStyleInfo feeHeaderStyleInfo = new Syncfusion.WinForms.DataGrid.Styles.HeaderStyleInfo();
            Syncfusion.WinForms.DataGrid.Styles.HeaderStyleInfo taxHeaderStyleInfo = new Syncfusion.WinForms.DataGrid.Styles.HeaderStyleInfo();
            /// color table : https://htmlcolorcodes.com/color-names/
            feeHeaderStyleInfo.BackColor = Color.FromKnownColor(KnownColor.LightSteelBlue);
            feeHeaderStyleInfo.TextColor = Color.White;
            taxHeaderStyleInfo.BackColor = Color.FromKnownColor(KnownColor.LightSlateGray);
            taxHeaderStyleInfo.TextColor = Color.White;

            dgvEditReceipts.Columns.Add(new GridNumericColumn() { MappingName = "FeePrice", HeaderText = "금액", Width = 118, HeaderStyle = feeHeaderStyleInfo });
            dgvEditReceipts.Columns.Add(new GridTextColumn() { MappingName = "TaxAbstract", HeaderText = "적요", Width = 118, HeaderStyle = taxHeaderStyleInfo });
            dgvEditReceipts.Columns.Add(new GridNumericColumn() { MappingName = "TaxPrice", HeaderText = "금액", Width = 118, HeaderStyle = taxHeaderStyleInfo });

            /// ref : https://help.syncfusion.com/windowsforms/sfdatagrid/columns#stacked-headers
            StackedHeaderRow shr = new StackedHeaderRow();
            shr.StackedColumns.Add(new StackedColumn() { ChildColumns = "FeeAbstract,FeePrice", HeaderText = "보수액" });
            shr.StackedColumns.Add(new StackedColumn() { ChildColumns = "TaxAbstract,TaxPrice", HeaderText = "공과금" });
            /// color table : https://htmlcolorcodes.com/color-names/
            dgvEditReceipts.StackedHeaderRows.Add(shr);
            dgvEditReceipts.Style.StackedHeaderStyle.BackColor = Color.SteelBlue;
            dgvEditReceipts.Style.StackedHeaderStyle.TextColor = Color.White;
            //= new Syncfusion.WinForms.DataGrid.Styles.StackedHeaderStyleInfo() { BackColor = Color.Black, TextColor = Color.White }; // bug

            /// Add Summary row;
            GridTableSummaryRow tableSummaryRow1 = new GridTableSummaryRow();
            tableSummaryRow1.Name = "TableSummary";
            tableSummaryRow1.ShowSummaryInRow = false;  // Row 전체를 사용하여 택스트를 표현하고자 할땐 True;
            tableSummaryRow1.Position = VerticalPosition.Bottom;

            GridSummaryColumn summaryColumn1 = new GridSummaryColumn();
            summaryColumn1.Name = "FeePrice";
            summaryColumn1.SummaryType = SummaryType.Int32Aggregate;
            summaryColumn1.Format = "{Sum:c}";
            summaryColumn1.MappingName = "FeePrice";
            GridSummaryColumn summaryColumn2 = new GridSummaryColumn();
            summaryColumn2.Name = "TaxPrice";
            summaryColumn2.SummaryType = SummaryType.Int32Aggregate;
            summaryColumn2.Format = "{Sum:c}";
            summaryColumn2.MappingName = "TaxPrice";

            tableSummaryRow1.SummaryColumns.Add(summaryColumn1);
            tableSummaryRow1.SummaryColumns.Add(summaryColumn2);
            dgvEditReceipts.TableSummaryRows.Add(tableSummaryRow1);

            /// Custom rendering;
            dgvEditReceipts.CellRenderers.Remove("TableSummary");
            dgvEditReceipts.CellRenderers.Add("TableSummary", new CustomGridTableSummaryRenderer());

            _ReceiptsTypesList = ReceiptDetType.GetSampleList();
            dgvEditReceipts.DataSource = _ReceiptsTypesList;
            dgvEditReceipts.LiveDataUpdateMode = LiveDataUpdateMode.AllowSummaryUpdate | LiveDataUpdateMode.AllowDataShaping;
            dgvEditReceipts.CurrentCellValidated += dgvEditReceipts_CurrentCellValidated;
        }

        void dgvEditReceipts_CurrentCellValidated(object sender, Syncfusion.WinForms.DataGrid.Events.CurrentCellValidatedEventArgs e)
        {
            var feePriceSV = SummaryCreator.GetSummaryDisplayText(dgvEditReceipts.View.Records.TableSummaries[0], "FeePrice", dgvEditReceipts.View);
            var taxPriceSV = SummaryCreator.GetSummaryDisplayText(dgvEditReceipts.View.Records.TableSummaries[0], "TaxPrice", dgvEditReceipts.View);
        }
    }

    public class CustomGridTableSummaryRenderer : GridTableSummaryCellRenderer
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="paint"></param>
        /// <param name="cellRect"></param>
        /// <param name="cellValue"></param>
        /// <param name="style"></param>
        /// <param name="column"></param>
        /// <param name="rowColumnIndex">Syncfusion.WinForms.GridCommon.ScrollAxis.RowColumnIndex</param>
        protected override void OnRender(Graphics paint, Rectangle cellRect, string cellValue, Syncfusion.WinForms.DataGrid.Styles.CellStyleInfo style, Syncfusion.WinForms.DataGrid.DataColumnBase column, Syncfusion.WinForms.GridCommon.ScrollAxis.RowColumnIndex rowColumnIndex)
        {
            if (string.IsNullOrEmpty(cellValue))
            {
                if (column.GridColumn.MappingName == "FeeAbstract" || column.GridColumn.MappingName == "TaxAbstract")
                {
                    cellValue = "소계";
                    StringFormat stringFormat1 = new StringFormat();
                    stringFormat1.LineAlignment = StringAlignment.Far;
                    stringFormat1.Alignment = StringAlignment.Near;
                    cellRect.X += 2;
                    paint.DrawString(cellValue, style.Font.GetFont(), Brushes.Black, cellRect, stringFormat1);
                }

                return;
            }
            // Creates new number format and apply it to summary value. 
            NumberFormatInfo format = new NumberFormatInfo();
            format.NumberDecimalDigits = 3;
            format.NumberDecimalSeparator = " * ";
            format.NumberGroupSeparator = ",";
            //Number format is applied to summary value.
            cellValue = Convert.ToInt32(double.Parse(cellValue, NumberStyles.Currency)).ToString("#,###", format);
            StringFormat stringFormat = new StringFormat();
            stringFormat.LineAlignment = StringAlignment.Far;
            stringFormat.Alignment = StringAlignment.Far;


            cellRect.Width -= 3;
            paint.DrawString(cellValue, style.Font.GetFont(), Brushes.Black, cellRect, stringFormat);
        }
    }


    public class ReceiptDetType : INotifyPropertyChanged
    {
        private string feeAbstract;
        private int feePrice;
        private string taxAbstract;
        private int taxPrice;
        public string FeeAbstract
        {
            get
            {
                return feeAbstract;
            }
            set
            {
                feeAbstract = value;
                OnPropertyChanged("FeeAbstract");
            }

        } // 보수액 적요;
        public int FeePrice 
        {
            get
            {
                return feePrice;
            }
            set
            {
                feePrice = value;
                OnPropertyChanged("FeePrice");
            }

        }   // 보수엑
        public string TaxAbstract 
        {
            get
            {
                return taxAbstract;
            }
            set
            {
                taxAbstract = value;
                OnPropertyChanged("TaxAbstract");
            }

        } // 공과세금 적요;
        public int TaxPrice 
        {
            get
            {
                return taxPrice;
            }
            set
            {
                taxPrice = value;
                OnPropertyChanged("TaxPrice");
            }

        }   // 공과세금;

        public ReceiptDetType(string feeAbstract, int feePrice, string taxAbstract, int taxPrice)
        {
            this.FeeAbstract = feeAbstract;
            this.FeePrice = feePrice;
            this.TaxAbstract = taxAbstract;
            this.TaxPrice = taxPrice;
        }

        static public List<ReceiptDetType> GetSampleList()
        {
            return (new List<ReceiptDetType>()
            {
                new ReceiptDetType("적요1", 10000, "세금1", 11000),
                new ReceiptDetType("적요2", 20000, "세금1", 11000),
                new ReceiptDetType("적요3", 30000, "세금1", 11000),
                new ReceiptDetType("적요4", 40000, "세금1", 41000),
                new ReceiptDetType("정요2", 10000, "세금1", 11000),
                new ReceiptDetType("적요1", 10000, "세금1", 11000)
            });
        }

        public event PropertyChangedEventHandler PropertyChanged;
        private void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

    }
}
